#ifndef SpatialPlotTool_h
#define SpatialPlotTool_h

#include "AbstractPlotTool.h"

#include <QMap>

class QAction;
class QComboBox;
class QMainWindow;

template <typename T> class QVector;

namespace Isis {
  class CubePlotCurve;
  class PlotWindow;
  class RubberBandComboBox;

  /**
   * @brief Spatial Plots
   *
   * This tool provides spatial plots for the user.
   *
   * @author ????-??-?? Stacy Alley
   *  
   * @internal 
   *   @history 2008-08-18 Christopher Austin - Upgraded to geos3.0.0
   *   @history 2008-09-05 Stacy Alley allowed spectral plotting of a single
   *                           point.
   *   @history 2009-01-29 Steven Lambright - Added RotatedRectangle to the
   *                           spatial plot
   *   @history 2010-06-26 Eric Hyer - Now uses MdiCubeViewport instead of
   *                           CubeViewport.  Fixed some include issues (many
   *                           still remain!).
   *   @history 2010-11-08 Eric Hyer - Spacial plot now handles linked images.
   *   @history 2011-03-18 Sharmila Prasad - Connect the viewport's close signal
   *   @history 2011-09-20 Steven Lambright - Now handles NULL statistical
   *                           values when graphing by not displaying them.
   *                           Fixes #234.
   */
  class SpatialPlotTool : public AbstractPlotTool {
      Q_OBJECT

    public:
      SpatialPlotTool(QWidget *parent);

    public slots:
      void refreshPlot();

    protected:
      QWidget *createToolBarWidget(QStackedWidget *parent);
      virtual PlotWindow *createWindow();
      virtual void detachCurves();
      void enableRubberBandTool();
      QAction *toolPadAction(ToolPad *pad);
      void updateTool();

    protected slots:
      virtual void rubberBandComplete();
      void viewportSelected();

    private:
      void getSpatialStatistics(QVector<double>  & labels,
                                QVector<double> & data, MdiCubeViewport *);
      void validatePlotCurves();
      double testSpecial(double pixel);

      //! Plot tool's action
      QPointer<QAction> m_toolPadAction;

      //! Allows the user to choose the interpolation type
      QPointer<QComboBox> m_interpolationCombo;

      //! Spatial curve
      QScopedPointer<
        QMap< MdiCubeViewport *, QPointer<CubePlotCurve> > > m_spatialCurves;

      //! Spatial plot rubber band combo box
      QPointer<RubberBandComboBox> m_rubberBandCombo;
  };
};

#endif
